const { Playlist } = require('../playlist')

const MAX_LENGTH = 2000

function playlist_to_discord_message(playlist, loop_name, { max_length = MAX_LENGTH } = {}) {
  if (playlist.is_empty()) {
    return string_to_discord_message("Empty playlist", {max_length})
  } else {
    let string = `Current playlist: (${playlist.get_name()}, looping: ${loop_name})\n`
        + "```json\n"
    const lines = playlist.map_tracks((t, id, _, current) => (current === t ? '(*) ' : '') + `${id + 1} ${t}`)
    string += array_to_discord_message(lines, {max_length: max_length - string.length - 3,
                                               index: playlist.get_current_index(),
                                               separator: "\n"})
      + '```'
    return string_to_discord_message(string, {max_length})
  }
}

// TODO: make limit more accurate
function array_to_discord_message(array, { max_length = MAX_LENGTH, index = 0, separator = "\n", limit = Infinity } = {}) {
  let string = '' + array[index]
  let i = 1
  let lines = 1
  while (string.length <= max_length
         && (array[index + i]
             || array[index - i])
         && lines < limit) {
    if (array[index + i]
        && string.length + (array[index + i] + '').length + separator.length <= max_length) {
      string += separator + array[index + i]
      lines += 1
    }
    if (array[index - i]
        && string.length + (array[index - i] + '').length + separator.length <= max_length) {
      string = array[index - i] + separator + string
      lines += 1
    }
    i += 1
  }
  return string_to_discord_message(string, { max_length })
}

function code_to_discord_message(message, language, { max_length = MAX_LENGTH } = {}) {
  return string_to_discord_message("```" + language + "\n" + message,
                                   { max_length: max_length - 3})
    + "```"
}

function string_to_discord_message(message, { max_length = MAX_LENGTH } = {}) {
  return message.slice(0, max_length)
}

module.exports = { playlist_to_discord_message,
                   string_to_discord_message,
                   array_to_discord_message,
                   MAX_LENGTH }
